# -*- coding: utf-8 -*-
import uno
import unohelper
from com.sun.star.task import XJobExecutor
import csv
import os
import logging
import webbrowser

from lookup import SphLookup
from lookup import get_syllable
from myhelper import MyUnoHelper

dir_path = os.path.dirname(os.path.realpath(__file__))

logging.basicConfig(filename=dir_path+'\\Addon.txt', level=logging.DEBUG)


def logException(name, e):
    logging.debug(name + "Exception " + str(type(e)) + " message " + str(e) + " args " + str(e.args))

def l(text):
    logging.debug(text)

class SphAnnotateJob(unohelper.Base, XJobExecutor):
    def __init__(self, ctx):
        # store the component context for later use
        self.ctx = ctx
        filepath = MyUnoHelper( ctx ).get_package_file( "addons.itjung.SimplePlaceholder", "phtab.pkl" )
        self.lookup = SphLookup( filepath )


    def writeTextToDocument(self, text) :

        #logging.debug("ich bin hier")
        oVC = self.helper.cursor()
        objText = self.helper.document().getText()
        #objCursor = objText.createTextCursor()

        objText.insertString(oVC, text, False)
        return


    def lookup_one(self,ch):
        return self.lookup.one(ch)

    def mark_textrange(self, oneSel):
        if not oneSel.supportsService("com.sun.star.text.TextRange"):
            return

        oText = oneSel.getText()
        oEnums = oneSel.createEnumeration()
        while oEnums.hasMoreElements():
            oPara = oEnums.nextElement()
            if not oPara.supportsService("com.sun.star.text.Paragraph"):
                continue
            oCursor = oText.createTextCursorByRange(oneSel.getStart())
            if oText.compareRegionStarts(oPara, oneSel) > 0:
                oCursor = oText.createTextCursorByRange(oneSel.getStart())
            else:
                oCursor = oText.createTextCursorByRange(oPara.getStart())

            if oText.compareRegionEnds(oPara, oneSel) < 0:
                oEnd = oText.createTextCursorByRange(oneSel.getEnd())
            else:
                oEnd = oText.createTextCursorByRange(oPara.getEnd())

            while oCursor.goRight(1, True) and oText.compareRegionEnds(oCursor, oEnd) >= 0:
                ch = ord(oCursor.String)
                if ch >= 0x4e00 and ch <= 0x9fff:
                    oCursor.RubyText = get_syllable(self.lookup.one(ch))
                oCursor.collapseToEnd()

    def mark_char(self, sym):
        vc = self.helper.cursor()
        if vc.goRight(1, True):
            vc.RubyText = get_syllable(sym)
            vc.collapseToStart()

    def mark_selected_text(self):
        if not self.helper.has_text_selection():
            return
        selection = self.helper.controller().getSelection()
        undo = self.helper.undomanager()
        undo.enterUndoContext("SphAnnotate")

        for i in range(selection.getCount()):
            self.mark_textrange(selection.getByIndex(i))

        undo.leaveUndoContext()

    def trigger(self, args):
        try:
            self.helper = MyUnoHelper( self.ctx )
            # Retrieve the desktop object
            if args.startswith("marksel"):
                self.mark_selected_text()
            elif args.startswith("premium"):
                url = 'https://itservicejung.de/simple-placeholder-premium/'
                webbrowser.open_new(url)
            elif args.startswith("change="):
                try:
                    editArray = self.helper.InputDoubleBox("Change your Placeholder", args.split("=")[2], args.split("=")[1])
                    menuindex =  args.split("=")[3]
                    l("Menuindex: "+menuindex)

                    if editArray[0] != "" and editArray[1] != "":

                        dir_path = os.path.dirname(os.path.realpath(__file__))

                        readerhandle = open(dir_path+ '\\eggs.csv')
                        writerhandle = open(dir_path+ '\\eggs_temp.csv', 'w', newline='')

                        reader = csv.reader(readerhandle)
                        writer = csv.writer(writerhandle)

                        row = 0
                        for line in reader:
                            if row == int(menuindex):
                               writer.writerow([str(editArray[0]), str(editArray[1])])
                            else:
                               writer.writerow([str(line[0]), str(line[1])])
                            row += 1
                        l(dir_path)
                        readerhandle.close();
                        writerhandle.close();
                        os.remove(dir_path+ '\\eggs.csv')
                        os.rename(dir_path+ '\\eggs_temp.csv', dir_path + '\\eggs.csv')
                except Exception as e:
                    logging.debug("dialog change exception="+str(e))

                #logging.debug(editArray)
            elif args.startswith("inputtext="):
                logging.debug("Input box start" + args.split("=")[1])
                self.writeTextToDocument(args.split("=")[1])
            else:
                logging.debug("trigger() other")
        except Exception as e:
            logException("trigger()", e)

# pythonloader looks for a static g_ImplementationHelper variable
g_ImplementationHelper = unohelper.ImplementationHelper()

g_ImplementationHelper.addImplementation( \
    SphAnnotateJob,  # UNO object class
    "addons.itjung.SimplePlaceholder.Job",  # Implementation name
    ("com.sun.star.task.Job",), )  # List of implemented services
